package org.gidea.scada.monitor.rabbit.listener;

import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.BuiltinExchangeType;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.gidea.scada.monitor.rabbit.entity.MonitorMessage;
import org.gidea.scada.monitor.rabbit.handler.RabbitMessageHandler;
import org.gidea.scada.monitor.utils.EmptyUtil;
import org.gidea.scada.monitor.utils.MonitorUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.ConcurrentReferenceHashMap;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.locks.Lock;

@Log4j2
@Component
public class MonitorRabbitListener {
    @Resource
    private RabbitMessageHandler messageHandler;

    @Value("${org.gidea.monitor.name}")
    private String monitorName;

    private Map<String, SimpleMessageListenerContainer> queues = new ConcurrentReferenceHashMap<>();

    private Map<String, Lock> queueLocks = new ConcurrentReferenceHashMap<>();

    @Resource
    private SimpleRabbitListenerContainerFactory containerFactory;
    @Resource
    private Channel channel;


    private void queueListener(String deviceId, SimpleMessageListenerContainer container, String responseQueueName) {
        container.setQueueNames(responseQueueName);
        container.setMessageListener(new ChannelAwareMessageListener() {
            @Override
            public void onMessage(Message message, Channel channel) {
                String messageStr = new String(message.getBody());
                if (messageStr.startsWith("{") && messageStr.endsWith("}")) {
                    MonitorMessage data = JSONObject.parseObject(messageStr, MonitorMessage.class);
                    if (EmptyUtil.isEmpty(data.getClientId()) || monitorName.equals(data.getClientId())) {
                        messageHandler.handleMessage(deviceId, messageStr);
                    }
                    return;
                }
                messageHandler.handleMessage(deviceId, messageStr);
            }
        });
        container.start();
    }

    public void addDeviceListener(String deviceId) {
        String queueName = MonitorUtils.getRequestQueueName(deviceId, monitorName);
        // 创建监听设备请求队列
        SimpleMessageListenerContainer container = queues.get(queueName);
        if (container != null) {
            return;
        }
        try {
            String exchangeName = MonitorUtils.getClientExchangeName(deviceId);
            channel.exchangeDeclare(exchangeName, BuiltinExchangeType.FANOUT, false, false, null);
            channel.queueDeclare(queueName, true, true, true, null);
            channel.queueBind(queueName, exchangeName, deviceId);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        container = containerFactory.createListenerContainer();
        queueListener(deviceId, container, queueName);
        queues.put(queueName, container);
    }

    public void removeDeviceListener(String queueResponseName) {
        // 删除监听设备响应队列
        try {
            SimpleMessageListenerContainer container = queues.get(queueResponseName);
            if (container == null) {
                return;
            }
            container.stop();
            queues.remove(queueResponseName);
        } finally {
            // 删除设备锁信息
            queueLocks.remove(queueResponseName);
        }
    }


}
