package org.gidea.scada.monitor.config;

import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.gidea.scada.monitor.utils.Constants;
import org.gidea.scada.monitor.utils.EmptyUtil;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.EnableRabbit;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.io.IOException;

@Log4j2
@EnableRabbit
@Configuration
public class RabbitMQConfig {

    @Value("${org.gidea.rabbitmq.host}")
    private String host;
    @Value("${org.gidea.rabbitmq.port}")
    private Integer port;
    @Value("${org.gidea.rabbitmq.username}")
    private String username;
    @Value("${org.gidea.rabbitmq.password}")
    private String password;

    @Resource
    private ApplicationContext applicationContext;

    @Bean
    public CachingConnectionFactory connectionFactory() {
        // 创建 ConnectionFactory 实例
        CachingConnectionFactory factory = new CachingConnectionFactory();
        // 设置 RabbitMQ 服务器相关参数
        factory.setHost(host); // 替换为实际的 RabbitMQ 主机地址
        factory.setPort(port); // 默认端口，如有变动请替换
        factory.setUsername(username); // 替换为实际的 RabbitMQ 用户名
        factory.setPassword(password); // 替换为实际的 RabbitMQ 密码
        factory.setVirtualHost("/"); // 替换为实际的 RabbitMQ 虚拟主机名称，默认为 /
        return factory;
    }


    @Bean
    public Channel channel(CachingConnectionFactory factory) {
        Channel channel = factory.createConnection().createChannel(false);
        return channel;
    }

    @Bean
    public SimpleRabbitListenerContainerFactory rabbitListenerContainerFactory(CachingConnectionFactory connectionFactory) {
        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        factory.setConnectionFactory(connectionFactory);
        // 可选配置：
        // 设置确认模式（默认为 AUTO_ACKNOWLEDGE）
        factory.setAcknowledgeMode(AcknowledgeMode.AUTO);
        // 设置消费者数量（默认为 1）
        factory.setConcurrentConsumers(5);
        // 设置最大消费者数量（默认为 Integer.MAX_VALUE）
        factory.setMaxConcurrentConsumers(10);
        return factory;
    }


    @Bean
    public RabbitAdmin amqpAdmin(CachingConnectionFactory connectionFactory) {
        RabbitAdmin admin = new RabbitAdmin(connectionFactory);
        return admin;
    }
}
