package org.gidea.scada.monitor.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.NameValuePair;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class HttpClient {

    private static Logger logger = LoggerFactory.getLogger(HttpClient.class);

    private static int TIME_OUT = 5;

    public static String doGet(String url, Map<String, String> haeder, Map<String, String> param) {
        return doGet(url, haeder, param, TIME_OUT);
    }

    public static String doGet(String url, Map<String, String> haeder, Map<String, String> param, int timeOut) {

        // 创建Httpclient对象
        CloseableHttpClient httpClient = HttpClients.createDefault();

        CloseableHttpResponse response = null;
        String resultString = "";
        try {
            URIBuilder builder = new URIBuilder(url);
            if (param != null) {
                Iterator var7 = param.keySet().iterator();
                while (var7.hasNext()) {
                    String key = (String) var7.next();
                    builder.addParameter(key, (String) param.get(key));
                }
            }
            URI uri = builder.build();
            // 创建Http Post请求
            HttpGet httpGet = new HttpGet(uri);
            if (haeder != null && haeder.keySet().size() > 0) {
                for (String head : haeder.keySet()) {
                    httpGet.setHeader(head, haeder.get(head));
                }
            }
            RequestConfig config = RequestConfig.custom().setConnectTimeout(timeOut * 1000).setSocketTimeout(timeOut * 1000).setConnectionRequestTimeout(timeOut * 1000).build();
            httpGet.setConfig(config);
            // 执行http请求
            response = httpClient.execute(httpGet);
            resultString = EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.error("访问GET地址报错：" + url);
            logger.error("访问地址Haeder：" + JSONObject.toJSONString(haeder));
            logger.error("访问地址参数：" + JSONObject.toJSONString(param));
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
            } catch (IOException e) {
            }
        }
        resultString = String.valueOf(JSON.parse(resultString));

        return resultString;
    }

    public static String doPost(String url, Map<String, String> haeder, Map<String, String> param) {

        // 创建Httpclient对象
        CloseableHttpClient httpClient = HttpClients.createDefault();

        CloseableHttpResponse response = null;
        String resultString = "";
        try {
            // 创建Http Post请求
            HttpPost httpPost = new HttpPost(url);
            if (haeder != null && haeder.keySet().size() > 0) {
                for (String head : haeder.keySet()) {
                    httpPost.setHeader(head, haeder.get(head));
                }
            }
            // 创建参数列表
            if (param != null) {
                List<NameValuePair> paramList = new ArrayList<>();
                for (String key : param.keySet()) {
                    String va1 = param.get(key);
                    paramList.add(new BasicNameValuePair(key, va1));
                }
                UrlEncodedFormEntity entity = new UrlEncodedFormEntity(paramList, "UTF-8");
                httpPost.setEntity(entity);
            }
            // 执行http请求
            response = httpClient.execute(httpPost);
            resultString = EntityUtils.toString(response.getEntity(), "utf-8");
        } catch (Exception e) {
            logger.error("访问POST地址报错：" + url);
            logger.error("访问地址Haeder：" + JSONObject.toJSONString(haeder));
            logger.error("访问地址参数：" + JSONObject.toJSONString(param));
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
            } catch (IOException e) {
            }
        }
        resultString = String.valueOf(JSON.parse(resultString));
        return resultString;
    }

    public static String doPostJson(String url, String json, Map<String, String> haeder) {
        String s = doPostJson(url, json, haeder, TIME_OUT);
        return s;
    }

    public static String doPostJson(String url, String json, Map<String, String> haeder, int timeOut) {

        // 创建Httpclient对象
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        String resultString = "";
        try {
            // 创建Http Post请求
            HttpPost httpPost = new HttpPost(url);
            if (haeder != null && haeder.keySet().size() > 0) {
                for (String head : haeder.keySet()) {
                    httpPost.setHeader(head, haeder.get(head));
                }
            }
            // 创建请求内容
            StringEntity entity = new StringEntity(json, ContentType.APPLICATION_JSON);
            RequestConfig config = RequestConfig.custom().setConnectTimeout(timeOut * 1000).setSocketTimeout(timeOut * 1000).setConnectionRequestTimeout(timeOut * 1000).build();
            httpPost.setConfig(config);
            httpPost.setEntity(entity);
            // 执行http请求
            response = httpClient.execute(httpPost);
            resultString = EntityUtils.toString(response.getEntity(), "UTF-8");
        } catch (Exception e) {
            logger.error("访问POST地址报错：" + url);
            logger.error("访问地址Haeder：" + JSONObject.toJSONString(haeder));
            logger.error("访问地址参数：" + json);
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
            } catch (IOException e) {
            }
        }
        return resultString;
    }
}
