package org.gidea.scada.monitor.mqtt.config;

import org.apache.activemq.ActiveMQConnection;
import org.apache.activemq.ActiveMQConnectionFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.jms.JMSException;
import javax.jms.Session;
import java.util.LinkedHashSet;
import java.util.Set;

@Configuration
public class MonitorMqttConfig {

    @Value("${org.gidea.mqtt.url}")
    private String url;

    @Value("${org.gidea.mqtt.username}")
    private String username;

    @Value("${org.gidea.mqtt.password}")
    private String password;

    @Bean(name = "mqttListeners")
    public Set<String> mqttListeners() {
        return new LinkedHashSet<>();
    }


    @Bean(name = "mqttConnection")
    public ActiveMQConnection connection() {
        ActiveMQConnectionFactory  factory = null;
        ActiveMQConnection connection = null;
        try {
            // 第一步：创建ConnectionFactory对象，需要指定服务端ip及端口号。
            factory = new ActiveMQConnectionFactory (username, password, url);
            // 第二步：使用ConnectionFactory对象创建一个Connection对象。
            connection = (ActiveMQConnection) factory.createConnection();
            // 第三步：开启连接，调用Connection对象的start方法。
            connection.start();
        } catch (JMSException e) {
            throw new RuntimeException(e);
        }
        return connection;
    }

    @Bean(name = "mqttSession")
    public Session session(ActiveMQConnection connection) {
        Session session = null;
        try {
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        } catch (JMSException e) {
            throw new RuntimeException(e);
        }
        return session;
    }

}
