package org.gidea.scada.monitor.mqtt.listener;

import com.rabbitmq.client.BuiltinExchangeType;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.apache.activemq.ActiveMQConnection;
import org.apache.activemq.advisory.DestinationSource;
import org.apache.activemq.command.ActiveMQTopic;
import org.gidea.scada.monitor.utils.MonitorUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.jms.*;
import java.util.Set;


@Log4j2
@Component
public class MonitorMqttListener {
    @Resource(name = "mqttConnection")
    private ActiveMQConnection connection;
    @Resource(name = "mqttSession")
    private Session session;
    @Resource(name = "mqttListeners")
    private Set<String> mqttListeners;
    @Resource
    private Channel channel;

    @Scheduled(fixedDelay = 10000)
    public void listener() throws Exception {
        DestinationSource destinationSource = connection.getDestinationSource();
        //获取有多少个主题
        Set<ActiveMQTopic> topics = destinationSource.getTopics();
        for (ActiveMQTopic topic : topics) {
            if (!mqttListeners.contains(topic.getTopicName())) {
                String deviceId = MonitorUtils.getDeviceIdByMqtt(topic.getTopicName());
                channel.exchangeDeclare(MonitorUtils.getClientExchangeName(deviceId), BuiltinExchangeType.FANOUT, false, false, null);
                mqttListeners.add(topic.getTopicName());
                //队列等待最多为1，后来的会替换新的
                Destination destination = session.createTopic(topic.getTopicName() + "?consumer.prefetchSize=1");
                // 第六步：使用Session对象创建一个Consumer对象。
                MessageConsumer consumer = session.createConsumer(destination);
                // 第七步：接收消息。
                consumer.setMessageListener(new MessageListener() {
                    @Override
                    public void onMessage(Message message) {
                        BytesMessage bm = (BytesMessage) message;
                        byte[] bys = null;
                        try {
                            //获取消息内容
                            bys = new byte[(int) bm.getBodyLength()];
                            bm.readBytes(bys);
                            String msg = new String(bys, "utf-8");
                            // 第八步：推送消息。
                            channel.basicPublish(MonitorUtils.getClientExchangeName(deviceId), deviceId, null, msg.toString().getBytes("UTF-8"));
                        } catch (Exception e) {
                            log.error("推送消息失败！", e);
                        }
                    }
                });
            }
        }
    }
}
